/*
 * Microstick demo with graphic LCD
 *  - PIC24HJ128 MCU
 *  - PCD8544 chipset, Nokia 3310 display
 *  - Graphic LCD library from agock.com
 */

//#define FOSC    (7370000ULL) // Fast RC internal oscillator 7.37 MHz

/* This is what we use: (FOSC defined in compiler options) */
//#define FOSC    (46062500ULL) // Fast RC osciilator with PLL, default settings: 7.37 MHz /2 *50 /4 = approx 46 MHz
#define FCY     (FOSC/2)

// SPI prescale is set at 4 (PIC24H.h)
// LCD SCK runs at FCY/4 Hz = approx 5.75 MHz

// LED on Microstick II board
#define LED  	 _LATA0
#define LED_TRIS _TRISA0

#include <stdint.h>
#include <xc.h>
#include <libpic30.h>
#include "main.h"
#include "glcd.h"

/* Global variables from other files */
extern volatile uint8_t unit_test_return; /* from glcd module */

/* Our global variables */
volatile uint32_t tick; /* this tick counts every 100ms */

/* Function prototype for Timer 1 ISR */
void __attribute__((__interrupt__, __auto_psv__)) _T1Interrupt(void);

/* Other function prototypes */
static void tick_init(void);

/* Timer 1 interrupt, also our tick timer */
void __attribute__((__interrupt__, __auto_psv__)) _T1Interrupt(void)
{
    /* Clear Timer 1 interrupt flag */
    _T1IF = 0;

	/* Increment tick counter */
	tick++;

    /* Toggle LED every second, just so we know this ISR is working */
	if (tick % 10 == 0) {
		LED ^= 1;
	}

	if (tick % 50 == 0) {
		/* Every 5s, change the glcd uint test */
		unit_test_return = 1; /* When set high, unit test will return */
	}
}

/* Initialise Timer 1 as our tick timer */
void tick_init(void) {
	/*
	 *  Configure Timer 1.
	 *
	 *  Tcy = 10.9ns
	 *  Period = PR1 * prescaler * Tcy = 9000 * 256 * 21.7ns = 100ms
	 */

	T1CON = 0;            // Clear Timer 1 configuration
	T1CONbits.TCKPS = 3;  // Set timer 1 prescaler (0=1:1, 1=1:8, 2=1:64, 3=1:256)
	PR1 = 9000;           // Set Timer 1 period (max value is 65535)
	_T1IP = 1;            // Set Timer 1 interrupt priority
	_T1IF = 0;            // Clear Timer 1 interrupt flag
	_T1IE = 1;            // Enable Timer 1 interrupt
	T1CONbits.TON = 1;    // Turn on Timer 1
}

int main() 
{
	/* Initialize the LED. This symbol is defined in system_config.h */
	LED_TRIS = 0;

	/* Initialise tick timer */
	tick_init();

	/* Initialise GLCD library */
	glcd_init();
	
	/* Run unit test demo, and switch every 5s (timer ISR takes care of this */
	while (1)
	{
		glcd_test_tiny_text();
		glcd_test_text_up_down();
		glcd_test_counter_and_graph();
		glcd_test_circles();
		glcd_test_scrolling_graph();
		glcd_test_glcdutils();
		glcd_test_rectangles();
	}

    return 0;
}
